from sklearn.neighbors import KNeighborsClassifier


class KNeighborsClassifierAlgorithm(KNeighborsClassifier):
    name = "knn"
    hyperparameters = [
        {
            "n_neighbors": [1, 2, 3, 4, 5, 6, 7, 8],
            "weights": ["uniform", "distance"],
            "p": [1, 2, 3, 4],
        }
    ]


KNNClassifier = KNeighborsClassifierAlgorithm()



from sklearn.svm import SVC
from sklearn.svm import LinearSVC
import numpy as np


class NaivelyCalibratedLinearSVC(LinearSVC):
    """LinearSVC with `predict_proba` method that naively scales
    `decision_function` output."""

    name = "lscv"

    def fit(self, X, y):
        super().fit(X, y)
        df = self.decision_function(X)
        self.df_min_ = df.min()
        self.df_max_ = df.max()

    def predict_proba(self, X):
        """Min-max scale output of `decision_function` to [0,1]."""
        df = self.decision_function(X)
        calibrated_df = (df - self.df_min_) / (self.df_max_ - self.df_min_)
        proba_pos_class = np.clip(calibrated_df, 0, 1)
        proba_neg_class = 1 - proba_pos_class
        proba = np.c_[proba_neg_class, proba_pos_class]
        return proba


class NaivelyCalibratedSVC(SVC):
    """LinearSVC with `predict_proba` method that naively scales
    `decision_function` output."""

    name = "svc"
    hyperparameters = [{"kernel": ["rbf"], "gamma": ["scale", "auto"]}]

    def fit(self, X, y):
        super().fit(X, y)
        df = self.decision_function(X)
        self.df_min_ = df.min()
        self.df_max_ = df.max()

    def predict_proba(self, X):
        """Min-max scale output of `decision_function` to [0,1]."""
        df = self.decision_function(X)
        calibrated_df = (df - self.df_min_) / (self.df_max_ - self.df_min_)
        proba_pos_class = np.clip(calibrated_df, 0, 1)
        proba_neg_class = 1 - proba_pos_class
        proba = np.c_[proba_neg_class, proba_pos_class]
        return proba


NaivelyCalibratedSVCLinear = NaivelyCalibratedLinearSVC(C=1.0)
NaivelyCalibratedSVCRBF = NaivelyCalibratedSVC(kernel="rbf", gamma=0.7, C=1.0)
NaivelyCalibratedSVCRBF.name = "rbf_svc"
NaivelyCalibratedSVCPolynomial = NaivelyCalibratedSVC(kernel="poly", degree=5, C=1.0)
NaivelyCalibratedSVCPolynomial.name = "poly_svc"



import os
import sys


sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))


from chembee_config.benchmark.BenchmarkAlgorithm import BenchmarkAlgorithm


class GridSearchCVClassifier(BenchmarkAlgorithm):

    name = "grid_search_cv"

    def __init__(self, clf_list: list, names: list):
        if len(names) != len(clf_list):
            raise ValueError("Len of classifiers and the name list must be equal.")
        self.algorithms = clf_list
        self.titles = names



import pandas as pd
from mordred import Calculator, descriptors

from rdkit import Chem
from rdkit.Chem import (
    Descriptors,
    Descriptors3D,
    rdMolDescriptors,
    PandasTools,
)

# Utilities
import os
from tqdm import tqdm


def calculate_mordred_descriptors(mols: list) -> pd.DataFrame:
    """
    The calculate_mordred_descriptors function calculates Mordred descriptors for a set of molecules.
    The function takes as input a list of RDKit molecule objects and returns the calculated descriptors in
    a pandas dataframe.

    :param mols: Used to Specify the molecules for which descriptors are to be calculated.
    :param : Used to Specify the calculation of 3d descriptors.
    :return: A pandas dataframe containing the calculated descriptors.

    :doc-author: Trelent
    """

    calc = Calculator(descriptors, ignore_3D=False)
    data = calc.pandas(mols)
    return data


def calculate_lipinski_desc(data_set: pd.DataFrame, mols: pd.Series) -> pd.DataFrame:
    """
    The calculate_lipinski_desc function calculates the molecular descriptors for each molecule in a data set.
    The function takes two arguments:
        1) data_set - A pandas DataFrame containing the SMILES strings of each molecule in a data set.
        2) mols - A pandas Series containing the RDKit Mol object of each molecule in a data set.

    The function returns one value:
        1) The original DataFrame with additional columns for each descriptor, which contain floats.

    :param data_set:pd.DataFrame: Used to Store the calculated descriptors.
    :param mols:pd.Series: Used to Pass a series of molecules to the function.
    :return: A dataframe with the descriptors for each molecule in the mols series.

    :doc-author: Trelent
    """

    for i, mol in tqdm(enumerate(mols)):
        Chem.SanitizeMol(mol)
        data_set.loc[i, "Molecule"] = mol
        data_set.loc[i, "MolWt"] = Descriptors.MolWt(mol)
        data_set.loc[i, "LogP"] = Descriptors.MolLogP(mol)
        data_set.loc[i, "NumHAcceptors"] = Descriptors.NumHAcceptors(mol)
        data_set.loc[i, "NumHDonors"] = Descriptors.NumHDonors(mol)
        data_set.loc[i, "NumHeteroatoms"] = Descriptors.NumHeteroatoms(mol)
        data_set.loc[i, "NumRotatableBonds"] = Descriptors.NumRotatableBonds(mol)
        data_set.loc[i, "NumHeavyAtoms"] = Descriptors.HeavyAtomCount(mol)
        data_set.loc[
            i, "NumAliphaticCarbocycles"
        ] = Descriptors.NumAliphaticCarbocycles(mol)
        data_set.loc[
            i, "NumAliphaticHeterocycles"
        ] = Descriptors.NumAliphaticHeterocycles(mol)
        data_set.loc[i, "NumAliphaticRings"] = Descriptors.NumAliphaticRings(mol)
        data_set.loc[i, "NumAromaticCarbocycles"] = Descriptors.NumAromaticCarbocycles(
            mol
        )
        data_set.loc[
            i, "NumAromaticHeterocycles"
        ] = Descriptors.NumAromaticHeterocycles(mol)
        data_set.loc[i, "NumAromaticRings"] = Descriptors.NumAromaticRings(mol)
        data_set.loc[i, "RingCount"] = Descriptors.RingCount(mol)
        data_set.loc[i, "FractionCSP3"] = Descriptors.FractionCSP3(mol)

        data_set.loc[i, "TPSA"] = Descriptors.TPSA(mol)
        data_set.loc[i, "NPR1"] = rdMolDescriptors.CalcNPR1(mol)
        data_set.loc[i, "NPR2"] = rdMolDescriptors.CalcNPR2(mol)
        data_set.loc[i, "InertialShapeFactor"] = Descriptors3D.InertialShapeFactor(mol)
        data_set.loc[i, "RadiusOfGyration"] = Descriptors3D.RadiusOfGyration(mol)
    return data_set


def load_data(file_path: str):
    """
    The load_data function loads the data from a file and returns it as a Pandas DataFrame.

    :param file_path:str: Used to Specify the location of the.
    :return: A dataframe with the following columns:.

    :doc-author: Trelent
    """

    sdfFile = os.path.join(file_path)
    mols = Chem.SDMolSupplier(sdfFile)
    frame = PandasTools.LoadSDF(
        sdfFile,
        smilesName="SMILES",
        molColName="Molecule",
        includeFingerprints=True,
        removeHs=False,
        strictParsing=True,
    )
    frame = calculate_lipinski_desc(data_set=frame, mols=mols)
    return frame


if __name__ == "__main__":
    from utils import save_csv

    data = load_data("tests/data/Biodeg.sdf")
    save_csv(data, "converted_data.csv")



from chembee_datasets.DataSet import DataSet
from sklearn import datasets
from sklearn.model_selection import train_test_split

from file_utils import prepare_file_name_saving


class BreastCancerDataset(DataSet):

    name = "breast-cancer"

    def __init__(self, split_ratio):

        self.data = self.load_data_set()
        self.split_ratio = split_ratio
        (
            self.X_train,
            self.X_test,
            self.y_train,
            self.y_test,
        ) = self.make_train_test_split(self.data, self.split_ratio)

    def load_data_set(self):
        """
        The load_data_set function loads the data from the csv file and creates a list of lists.
        The function also removes any rows with missing values, as well as any columns that have all zeros.
        The function returns a tuple containing two elements: (data_set, target)

        :param self: Used to Reference the class object.
        :return: The cancer_data dataframe.

        :doc-author: Trelent
        """

        cancer_data = datasets.load_breast_cancer()
        return cancer_data

    def make_train_test_split(self, data, split_ratio, shuffle=True):
        """
        The make_train_test_split function splits the data into training and testing sets.

        Parameters:
            data (object): The dataset to be split.

            split_ratio (float): The ratio of the number of training samples to total number of samples in the dataset.

            shuffle (bool, optional): Whether or not to shuffle the input before splitting it into train and test sets. Defaults to True if not specified otherwise.

        :param self: Used to Reference the class instance.
        :param data: Used to Pass the data set to be split.
        :param split_ratio: Used to determine the ratio of samples used for training.
        :param shuffle=True: Used to shuffle the data before splitting it into train and test sets.
        :return: The following:.

        :doc-author: Trelent
        """

        X = data.data[:, :2]
        y = data.target
        train_samples = int(split_ratio * len(X))
        X_train, X_test, y_train, y_test = train_test_split(
            X,
            y,
            shuffle=shuffle,
            test_size=len(X) - train_samples,
        )
        return X_train, X_test, y_train, y_test

    def save_data_npy(self, data, file_name, prefix=None):
        import numpy as np

        file_name = prepare_file_name_saving(
            prefix=prefix, file_name=file_name, ending=".npy"
        )
        np.save(file_name, data)



import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))


from sklearn.cluster import SpectralClustering


from chembee_config.benchmark.BenchmarkAlgorithm import BenchmarkAlgorithm


class SpectralClusteringClassifier(BenchmarkAlgorithm):

    name = "spectral-clustering"
    C = 1.0  # SVM regularization parameter
    algorithms = (
        SpectralClustering(
            n_clusters=3,
            eigen_tol=1e-7,
            assign_labels="kmeans",
            random_state=42,
        ),
        SpectralClustering(
            n_clusters=3,
            eigen_tol=1e-7,
            assign_labels="discretize",
            random_state=42,
        ),
        SpectralClustering(
            n_clusters=3,
            eigen_tol=1e-7,
            assign_labels="cluster_qr",
            random_state=42,
        ),
    )
    titles = (
        "k-means",
        "discretize method",
        "cluster_qr",
    )
    _response_method = "fit_predict"



from chembee_config.calibration.logistic_regression import LogisticRegressionClassifier
from chembee_config.calibration.naive_bayes import NaiveBayesClassifier
from chembee_config.calibration.svc import NaivelyCalibratedSVCRBF
from chembee_config.calibration.svc import NaivelyCalibratedSVCPolynomial
from chembee_config.calibration.random_forest import RandomForestClassifier
from chembee_config.calibration.knn import KNNClassifier
from chembee_config.calibration.mlp_classifier import NeuralNetworkClassifierRELU
from chembee_config.calibration.mlp_classifier import NeuralNetworkClassifierTanh


clf_list = [
    LogisticRegressionClassifier,
    NaiveBayesClassifier,
    NaivelyCalibratedSVCRBF,
    NaivelyCalibratedSVCPolynomial,
    RandomForestClassifier,
    KNNClassifier,
    NeuralNetworkClassifierRELU,
    NeuralNetworkClassifierTanh,
]



import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))


from sklearn import svm

from chembee_config.benchmark.BenchmarkAlgorithm import BenchmarkAlgorithm


class SVClassifier(BenchmarkAlgorithm):

    name = "svc"
    C = 1.0  # SVM regularization parameter
    algorithms = (
        svm.SVC(kernel="linear", C=C),
        svm.LinearSVC(C=C, max_iter=10000),
        svm.SVC(kernel="rbf", gamma=0.7, C=C),
        svm.SVC(kernel="poly", degree=3, gamma="auto", C=C),
    )
    titles = (
        "SVC with linear kernel",
        "LinearSVC (linear kernel)",
        "SVC with RBF kernel",
        "SVC with polynomial (degree 3) kernel",
    )



import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
from math import pi

from file_utils import prepare_file_name_saving
from chembee_datasets.DataSet import DataSet


def polar_plot(data_set: DataSet, file_name: str, prefix: str):
    """
    The polar_plot function creates a polar plot of the lipinski parameters.
    It takes in a dataframe and file name as input, and outputs a png file with the
    polar plot.

    :param data_set:pd.DataFrame: Used to Set the data that will be plotted.
    :param file_name:str: Used to Create a directory for each molecule.
    :param prefix:str: Used to Determine the name of the file.
    :return: The figure object.

    :doc-author: Julian M. Kleber
    """

    data_set_name = data_set.name
    data_set = data_set.data
    data = pd.DataFrame()

    # calculate normalized lipinski

    data["MolWt"] = [i / 500 for i in data_set["MolWt"]]
    data["LogP"] = [i / 5 for i in data_set["LogP"]]
    data["HBA"] = [i / 10 for i in data_set["NumHAcceptors"]]
    data["HBD"] = [i / 5 for i in data_set["NumHDonors"]]
    data["RotB"] = [i / 10 for i in data_set["NumRotatableBonds"]]
    data["TPSA"] = [i / 140 for i in data_set["TPSA"]]
    categories = list(
        data.columns
    )  # This will set up the parameters for the angles of the radar plot.
    N = len(categories)
    values = data[categories].values[0]
    values = np.append(values, values[:1])
    angles = [n / float(N) * 2 * pi for n in range(N)]
    angles += angles[:1]

    Ro5_up = [1, 1, 1, 1, 1, 1, 1]  # The upper limit for bRo5
    Ro5_low = [0.5, 0.1, 0, 0.25, 0.1, 0.5, 0.5]  # The lower limit for bRo5
    # fig=plt.figure()

    fig, ax = plt.subplots(subplot_kw={"projection": "polar"})

    plt.xticks(
        angles[:-1],
        categories,
        color="k",
        size=20,
        ha="center",
        va="top",
        fontweight="book",
    )

    plt.tick_params(axis="y", width=4, labelsize=12, grid_alpha=0.05)

    ax.set_rlabel_position(0)

    # ax.fill(angles, Ro5_up, 'red', alpha=0.2)
    ax.fill(angles, Ro5_low, "orangered", alpha=0.2)
    mean_vals = data.mean().to_list()
    mean_vals = np.append(mean_vals, mean_vals[:1])
    for i in data.index[:]:
        values = data[categories].values[i]
        values = np.append(values, values[:1])
        ax.plot(angles, values, linewidth=0.7, color="steelblue", alpha=0.5)
        # ax.fill(angles, values, 'C2', alpha=0.025)
    ax.plot(angles, mean_vals, linewidth=2, linestyle="--", color="red")
    ax.plot(angles, Ro5_up, linewidth=2, linestyle="-", color="black")
    ax.plot(angles, Ro5_low, linewidth=2, linestyle="-", color="black")

    ax.grid(axis="y", linewidth=1.5, linestyle="dotted", alpha=0.8)
    ax.grid(axis="x", linewidth=2, linestyle="-", alpha=1)
    file_name = prepare_file_name_saving(
        prefix=prefix, file_name=file_name, ending=".png"
    )
    fig.tight_layout()
    plt.savefig(file_name)



import matplotlib.pyplot as plt

import logging


logging.basicConfig(
    format="%(levelname)s:%(asctime)s %(message)s",
    datefmt="%m/%d/%Y %I:%M:%S %p",
    level=logging.INFO,
    filename="chembee_plotting.log",
)


from file_utils import get_grid_positions, prepare_file_name_saving


def plot_calibration(
    fig: plt.figure,
    clf_list: list,
    calibration_displays: dict,
    ax_calibration_curve,
    grid_spec,
    grid: tuple,
    colors,
    file_name: str = "calibration",
    prefix: str = "calibration/",
):

    ax_calibration_curve.grid()
    ax_calibration_curve.set_title("Calibration plots")
    # call other method to avoid messing it up

    grid_positions = get_grid_positions(rows=grid[0], cols=grid[1])
    for i in range(len(clf_list)):
        clf = clf_list[i]
        name = clf.name
        row, col = grid_positions[i]
        ax = fig.add_subplot(grid_spec[row, col])

        ax.hist(
            calibration_displays[name].y_prob,
            range=(0, 1),
            bins=100,
            label=name,
            color=colors(i),
        )
        if i == grid[0] - 1:
            ax.set(xlabel="Mean predicted probability", ylabel="Count")
        else:
            ax.set(title="", xlabel="", ylabel="Count")

    file_name = prepare_file_name_saving(prefix, file_name, ending=".png")
    fig.tight_layout()
    plt.savefig(file_name)
    logging.info("Saved calibration plot " + str(file_name))



import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))


from sklearn.naive_bayes import GaussianNB, MultinomialNB, BernoulliNB, ComplementNB

from chembee_config.benchmark.BenchmarkAlgorithm import BenchmarkAlgorithm


class NaiveBayesClassifier(BenchmarkAlgorithm):
    name = "naive-bayes"

    algorithms = (
        GaussianNB(),
        MultinomialNB(),
        BernoulliNB(),
        ComplementNB(),
    )

    titles = ("Gaussian", "Multinomial", "Bernoulli", "Complement")
